@@ -15,6 +15,7 @@ import pyubx2.ubxtypes_core as ubt
 import pyubx2.ubxtypes_get as ubg
 import pyubx2.ubxtypes_set as ubs
 import pyubx2.ubxtypes_poll as ubp
+import pyubx2.ubxtypes_configdb as ubcdb
 
 
 class UBXMessage:
@@ -67,10 +68,11 @@ class UBXMessage:
         self._immutable = True
 
     def _do_attributes(self, **kwargs):
-        """Populate UBXMessage from named attribute keywords.
+        """
+        Populate UBXMessage from named attribute keywords.
         Where a named attribute is absent, set to a nominal value (zeros or blanks).
 
-        :param **kwargs:
+        :param **kwargs: payload
 
         """
 
@@ -82,7 +84,7 @@ class UBXMessage:
                 self._payload = None
             else:
                 pdict = self._get_dict(**kwargs)  # get appropriate payload dict
-                for key in pdict.keys():  # set each attribute in dict
+                for key in pdict.keys():  # process each attribute in dict
                     (offset, att) = self._set_attribute(offset, pdict, key, **kwargs)
             self._do_len_checksum()
 
@@ -104,7 +106,8 @@ class UBXMessage:
     def _set_attribute(
         self, offset: int, pdict: dict, key: str, **kwargs
     ) -> (int, str):
-        """Recursive routine to populate individual payload attributes
+        """
+        Recursive routine to populate individual payload attributes
 
         :param offset: int:
         :param pdict: dict:
@@ -114,107 +117,111 @@ class UBXMessage:
         """
         # pylint: disable=no-member
 
-        #  if repeating group, suffix keyword with index
-        if self._index > 0:
+        # if within repeating group, suffix keyword with index
+        # (ESF-MEAS is a special case; currently the only supported message type
+        # which has an optional attribute (calibTtag) _after_ a repeating group)
+        if self._index > 0 and key != "calibTtag":
             keyr = key + "_{0:0=2d}".format(self._index)
         else:
             keyr = key
 
         att = pdict[key]  # get attribute type
-        if isinstance(att, tuple):  # attribute is a tuple i.e. a nested repeating group
+        if isinstance(att, tuple):  # repeating group of attributes
             numr, attd = att
-            if numr == "None":
-                rng = self._calc_num_repeats(attd, self._payload, offset)
+            # if CFG-VALGET message, use dedicated method to
+            # parse as configuration key value pairs
+            if (
+                self._ubxClass == b"\x06"
+                and self._ubxID == b"\x8b"
+                and self._mode == ubt.GET
+            ):
+                self._set_cfgval_attributes(offset, **kwargs)
             else:
-                rng = getattr(self, numr)
-            for i in range(rng):
-                self._index = i + 1
-                for key1 in attd.keys():
-                    (offset, _) = self._set_attribute(offset, attd, key1, **kwargs)
+                # derive or retrieve number of attributes in group
+                if numr == "None":
+                    rng = self._calc_num_repeats(attd, self._payload, offset)
+                elif (
+                    numr == "ESF-MEAS-CT"
+                ):  # special handling for ESF-MEAS message type
+                    rng = self._calc_num_repeats(attd, self._payload, offset, 4)
+                else:
+                    rng = getattr(self, numr)
+                # recursively process each attribute, incrementing the payload offset as we go
+                for i in range(rng):
+                    self._index = i + 1
+                    for key1 in attd.keys():
+                        (offset, _) = self._set_attribute(offset, attd, key1, **kwargs)
 
-        else:
+        else:  # single attribute
 
-            if att == ubt.CH:  # INF message payload
+            # determine attribute size (bytes)
+            if (
+                att == ubt.CH
+            ):  # payload is a single variable length string (e.g. INF message)
                 atts = len(self._payload)
             else:
                 atts = int(att[1:3])
 
-            # if the entire payload has been provided,
-            # use the appropriate section of the payload
+            # if payload keyword has been provided,
+            # use the appropriate offset of the provided payload
             if "payload" in kwargs:
                 self._payload = kwargs["payload"]
-                val = self._payload[offset : offset + atts]
-                if (
-                    att == ubt.CH
-                ):  # attribute is a single variable-length string (e.g. INF-NOTICE)
-                    val = self._payload.decode("utf-8", "backslashreplace")
-                elif att[0:1] in ("X", "C"):
-                    pass
-                elif att[0:1] == "U":  # unsigned integer
-                    val = int.from_bytes(val, "little", signed=False)
-                elif att[0:1] == "I":  # signed integer
-                    val = int.from_bytes(val, "little", signed=True)
-                elif att == ubt.R4:  # single precision floating point
-                    val = self.bytes_to_float(val)
-                elif att == ubt.R8:  # double precision floating point
-                    val = self.bytes_to_double(val)
+                if att == ubt.CH:
+                    valb = self._payload
                 else:
-                    raise ube.UBXTypeError(
-                        f"Unknown attribute type {att} for key {key}"
-                    )
+                    valb = self._payload[offset : offset + atts]
+                val = self.bytes2val(valb, att)
 
-            # else if individual attribute has been provided
-            elif keyr in kwargs:
-                val = kwargs[keyr]
-                if att[0:1] in ("X", "C"):  # byte or char
-                    valb = val
-                elif att[0:1] == "U":  # unsigned integer
-                    valb = val.to_bytes(atts, byteorder="little", signed=False)
-                elif att[0:1] == "I":  # signed integer
-                    valb = val.to_bytes(atts, byteorder="little", signed=True)
-                elif att == ubt.R4:  # single precision floating point
-                    valb = self.float_to_bytes(val)
-                elif att == ubt.R8:  # double precision floating point
-                    valb = self.double_to_bytes(val)
-                else:
-                    raise ube.UBXTypeError(
-                        f"Unknown attribute type {att} for key {key}"
-                    )
-                self._payload += valb
-
-            # else set individual attribute to nominal value
             else:
-                if att[0:1] in ("X", "C"):  # byte or char
-                    valb = b"\x00" * atts
-                    val = valb
-                elif att[0:1] == "U":  # unsigned integer
-                    val = 0
-                    valb = val.to_bytes(atts, byteorder="little", signed=False)
-                elif att[0:1] == "I":  # signed integer
-                    val = 0
-                    valb = val.to_bytes(atts, byteorder="little", signed=True)
-                elif att == ubt.R4:  # single precision floating point
-                    val = 0.0
-                    valb = self.float_to_bytes(val)
-                elif att == ubt.R8:  # double precision floating point
-                    val = 0.0
-                    valb = self.double_to_bytes(val)
+                # if individual attribute keyword has been provided
+                if keyr in kwargs:
+                    val = kwargs[keyr]
+                # else set attribute to nominal value (0)
                 else:
-                    raise ube.UBXTypeError(
-                        f"Unknown attribute type {att} for key {key}"
-                    )
+                    if att[0:1] in ("X", "C"):  # byte or char
+                        val = b"\x00" * atts
+                    else:
+                        val = 0
+                valb = self.val2bytes(val, att)
                 self._payload += valb
 
-        if not isinstance(att, tuple):
-            if self._index > 0:  # add 2-digit suffix to repeating attribute names
-                key = key + "_{0:0=2d}".format(self._index)
-            setattr(self, key, val)
+            setattr(self, keyr, val)
             offset += atts
 
         return (offset, att)
 
+    def _set_cfgval_attributes(self, offset: int, **kwargs):
+        """
+        Parse CFG-VALGET payload to set of configuration
+        key value pairs
+
+        :param offset int: payload offset
+        :param **kwargs  payload
+        """
+
+        self._payload = kwargs["payload"]
+        cfglen = len(self._payload[offset:])
+
+        i = 0
+        while offset < cfglen:
+            if i == 4:
+                key = int.from_bytes(
+                    self._payload[offset : offset + 4], "little", signed=False
+                )
+                (keyname, att) = self.cfgkey2name(key)
+                atts = int(att[1:3])
+                valb = self._payload[offset + 4 : offset + 4 + atts]
+                val = self.bytes2val(valb, att)
+                setattr(self, keyname, val)
+                i = 0
+                offset += 4 + atts
+
+            else:
+                i += 1
+
     def _do_len_checksum(self):
-        """Calculate and format payload length and checksum as bytes"""
+        """
+        Calculate and format payload length and checksum as bytes"""
 
         if self._payload is None:
             self._length = self.len2bytes(0)
@@ -228,8 +235,12 @@ class UBXMessage:
             )
 
     def _get_dict(self, **kwargs) -> dict:
-        """Get payload dictionary corresponding to message mode
+        """
+        Get payload dictionary corresponding to message mode (GET/SET/POLL)
+        MGA and older NMEA messages need special handling as their dictionary keys (ubxClass/ubxID)
+        are not unique
 
+        :param **kwargs payload
         :return dict:
 
         """
@@ -241,20 +252,25 @@ class UBXMessage:
                 pdict = self._get_mga_version(ubt.SET, **kwargs)
             else:
                 pdict = ubs.UBX_PAYLOADS_SET[self.identity]
-        else:
+        else:  # GET message
             if self._ubxClass == b"\x13" and self._ubxID != b"\x80":  # MGA GET message
                 pdict = self._get_mga_version(ubt.GET, **kwargs)
-            elif self.identity == "CFG-NMEA":
+            #             elif self.identity == "CFG-NMEA":
+            elif self._ubxClass == b"\x06" and self._ubxID == b"\x17":  # CFG-NMEA
                 pdict = self._get_cfgnmea_version(**kwargs)
+            elif self._ubxClass == b"\x10" and self._ubxID == b"\x02":  # ESF-MEAS
+                pdict = self._get_esfmeas_version(**kwargs)
             else:
                 pdict = ubg.UBX_PAYLOADS_GET[self.identity]
         return pdict
 
-    def _get_mga_version(self, mode, **kwargs) -> dict:
+    def _get_mga_version(self, mode: int, **kwargs) -> dict:
         """
         Select appropriate MGA payload definition
 
-         :return dict:
+        :param mode str: 0=GET, 1=SET, 2=POLL
+        :param **kwargs payload
+        :return dict:
 
         """
 
@@ -268,9 +284,10 @@ class UBXMessage:
 
     def _get_cfgnmea_version(self, **kwargs) -> dict:
         """
-        Selects appropriate payload definition version for older
+        Select appropriate payload definition version for older
         generations of CFG-NMEA message
 
+        :param **kwargs payload
         :return dict:
 
         """
@@ -284,29 +301,52 @@ class UBXMessage:
             pdict = ubg.UBX_PAYLOADS_GET["CFG-NMEA"]
         return pdict
 
-    def _calc_num_repeats(self, att, payload: bytes, offset: int) -> int:
-        """Deduce number of items in repeating group by dividing length of
+    def _get_esfmeas_version(self, **kwargs) -> dict:
+        """
+        Select appropriate payload definition version for
+        ESF-MEAS message by checking bit 3 (calibTtagValid)
+        in the'flags' attribute.
+
+        :param **kwargs payload
+        :return dict:
+        """
+
+        flags = kwargs["payload"][4:6]  # bytes
+        flags = int(flags.hex(), 16)  # int
+        calibTtagValid = flags >> 3 & 1  # test bit 3
+        if calibTtagValid:
+            pdict = ubg.UBX_PAYLOADS_GET["ESF-MEAS-CT"]
+        else:
+            pdict = ubg.UBX_PAYLOADS_GET["ESF-MEAS"]
+        return pdict
+
+    def _calc_num_repeats(
+        self, att, payload: bytes, offset: int, offsetend: int = 0
+    ) -> int:
+        """
+        Deduce number of items in repeating group by dividing length of
         remaining payload by length of group.
 
-        NB: this assumes the repeating group is always at the end of the
-        payload, which is true for all currently supported message types
-        but may change in the future.
+        This assumes there is only one such repeating group per message,
+        payload, which is true for all currently supported types.
 
-        :param att:
+        :param att: attribute type
         :param payload : bytes:
-        :param offset: int:
+        :param offset: int: number of bytes in payload before repeating group
+        :param offsetend: int: number of bytes in payload after repeating group
 
         """
         # pylint: disable=no-self-use
 
-        lenpayload = len(payload) - offset
+        lenpayload = len(payload) - offset - offsetend
         lengroup = 0
         for _, val in att.items():
             lengroup += int(val[1:3])
         return int(lenpayload / lengroup)
 
     def __str__(self) -> str:
-        """Human readable representation.
+        """
+        Human readable representation.
 
         :return: str:
 
@@ -354,7 +394,8 @@ class UBXMessage:
         return stg
 
     def __repr__(self) -> str:
-        """Machine readable representation.
+        """
+        Machine readable representation.
 
         eval(repr(obj)) = obj
 
@@ -369,6 +410,9 @@ class UBXMessage:
     def __setattr__(self, name, value):
         """
         Override setattr to make object immutable after instantiation
+
+        :param name
+        :param value
         """
 
         if self._immutable:
@@ -379,7 +423,8 @@ class UBXMessage:
         super().__setattr__(name, value)
 
     def serialize(self) -> bytes:
-        """Serialize message
+        """
+        Serialize message
 
         :return bytes:
 
@@ -411,15 +456,19 @@ class UBXMessage:
 
         """
 
-        # all MGA messages except MGA-DBD need to be identified by the
-        # 'type' attribute - the first byte of the payload
-        if self._ubxClass == b"\x13" and self._ubxID != b"\x80":
-            umsg_name = ubt.UBX_MSGIDS[
-                self._ubxClass + self._ubxID + self._payload[0:1]
-            ]
-        else:
-            umsg_name = ubt.UBX_MSGIDS[self._ubxClass + self._ubxID]
-
+        try:
+            # all MGA messages except MGA-DBD need to be identified by the
+            # 'type' attribute - the first byte of the payload
+            if self._ubxClass == b"\x13" and self._ubxID != b"\x80":
+                umsg_name = ubt.UBX_MSGIDS[
+                    self._ubxClass + self._ubxID + self._payload[0:1]
+                ]
+            else:
+                umsg_name = ubt.UBX_MSGIDS[self._ubxClass + self._ubxID]
+        except KeyError as err:
+            raise ube.UBXMessageError(
+                f"Unknown UBX message type class {self._ubxClass} id {self._ubxID}"
+            ) from err
         return umsg_name
 
     @property
@@ -438,9 +487,9 @@ class UBXMessage:
         return self._ubxID
 
     @property
-    def length(self) -> bytes:
-        """Payload length getter (as 2 little-endian bytes)"""
-        return self._length
+    def length(self) -> int:
+        """Payload length getter"""
+        return self.bytes2len(self._length)
 
     @property
     def payload(self) -> bytes:
@@ -449,13 +498,15 @@ class UBXMessage:
 
     @staticmethod
     def parse(message: bytes, validate: bool = False) -> object:
-        """Parse UBX byte stream to UBXMessage object.
+        """
+        Parse UBX byte stream to UBXMessage object.
 
         Includes option to validate incoming payload length and checksum
         (UXBMessage will calculate and assign it's own values anyway).
 
         :param message: bytes:
         :param validate: bool:  (Default value = False)
+        :return UBXMessage object:
 
         """
 
@@ -497,7 +548,8 @@ class UBXMessage:
 
     @staticmethod
     def msgclass2bytes(msgClass: int, msgID: int) -> bytes:
-        """Convert message class/id integers to bytes
+        """
+        Convert message class/id integers to bytes
         e.g. 6, 1 to b'/x06/x01'.
 
         :param msgClass: int:
@@ -511,7 +563,8 @@ class UBXMessage:
 
     @staticmethod
     def msgstr2bytes(msgClass: str, msgID: str) -> bytes:
-        """Convert plain text UBX message class to bytes
+        """
+        Convert plain text UBX message class to bytes
         e.g. 'CFG-MSG' to b'/x06/x01'.
 
         :param msgClass: str:
@@ -521,7 +574,6 @@ class UBXMessage:
 
         try:
             clsid = UBXMessage.key_from_val(ubt.UBX_CLASSES, msgClass)
-            # msgid = UBXMessage.key_from_val(ubt.UBX_MSGIDS, msgID)[1:]
             msgid = UBXMessage.key_from_val(ubt.UBX_MSGIDS, msgID)[1:2]
             return (clsid, msgid)
         except ube.UBXMessageError as err:
@@ -530,52 +582,60 @@ class UBXMessage:
             ) from err
 
     @staticmethod
-    def bytes_to_float(b: bytes) -> float:
-        """Convert 4 little-endian bytes (R4) to single precision decimal (IEEE754)
-
-        :param b: bytes:
-        :return float:
+    def val2bytes(val, att: str) -> bytes:
+        """
+        Return bytes from value for given attribute type
 
+        :param val value
+        :param att str: attribute type
+        :return bytes: value as bytes
         """
 
-        return struct.unpack("<f", b)[0]
+        atts = int(att[1:3])
+        if att[0:1] in ("C", "X"):  # byte or char
+            valb = val
+        elif att[0:1] in ("E", "L", "U"):  # unsigned integer
+            valb = val.to_bytes(atts, byteorder="little", signed=False)
+        elif att[0:1] == "I":  # signed integer
+            valb = val.to_bytes(atts, byteorder="little", signed=True)
+        elif att == ubt.R4:  # single precision floating point
+            valb = struct.pack("<f", val)
+        elif att == ubt.R8:  # double precision floating point
+            valb = struct.pack("<d", val)
+        else:
+            raise ube.UBXTypeError(f"Unknown attribute type {att}")
+        return valb
 
     @staticmethod
-    def bytes_to_double(b: bytes) -> float:
-        """Convert 8 little-endian bytes (R8) to double precision decimal (IEEE754)
-
-        :param b: bytes:
-        :return float:
+    def bytes2val(valb: bytes, att: str) -> object:
+        """
+        Return value from bytes for given attribute type
 
+        :param valb bytes: value in byte format
+        :param att str: attribute type
+        :return object: value
         """
 
-        return struct.unpack("<d", b)[0]
-
-    @staticmethod
-    def float_to_bytes(f: float) -> bytes:
-        """Convert single precision decimal to 4 little-endian bytes (R4) (IEEE754)
-
-        :param f: float:
-        :return float:
-
-        """
-
-        return struct.pack("<f", f)
-
-    @staticmethod
-    def double_to_bytes(f: float) -> bytes:
-        """Convert double precision decimal to 8 little-endian bytes (R8) (IEEE754)
-
-        :param f: float:
-        :return bytes:
-
-        """
-
-        return struct.pack("<d", f)
+        if att == ubt.CH:  # single variable-length string (e.g. INF-NOTICE)
+            val = valb.decode("utf-8", "backslashreplace")
+        elif att[0:1] in ("X", "C"):
+            val = valb
+        elif att[0:1] == "U":  # unsigned integer
+            val = int.from_bytes(valb, "little", signed=False)
+        elif att[0:1] == "I":  # signed integer
+            val = int.from_bytes(valb, "little", signed=True)
+        elif att == ubt.R4:  # single precision floating point
+            val = struct.unpack("<f", valb)[0]
+        elif att == ubt.R8:  # double precision floating point
+            val = struct.unpack("<d", valb)[0]
+        else:
+            raise ube.UBXTypeError(f"Unknown attribute type {att}")
+        return val
 
     @staticmethod
     def bytes2len(length: bytes) -> int:
-        """Convert payload length as bytes to integer.
+        """
+        Convert payload length as bytes to integer.
 
         :param length: bytes:
         :return int:
@@ -586,7 +646,8 @@ class UBXMessage:
 
     @staticmethod
     def len2bytes(length: int) -> bytes:
-        """Convert payload length as integer to two little-endian bytes.
+        """
+        Convert payload length as integer to two little-endian bytes.
 
         :param length: int:
         :return bytes:
@@ -617,7 +678,8 @@ class UBXMessage:
 
     @staticmethod
     def isvalid_checksum(message: bytes) -> bool:
-        """Validate input message's checksum
+        """
+        Validate input message's checksum
         ('message' includes header and checksum)
 
         :param message: bytes:
@@ -631,7 +693,8 @@ class UBXMessage:
 
     @staticmethod
     def itow2utc(iTOW: int) -> datetime.time:
-        """Convert UBX iTOW to UTC time
+        """
+        Convert UBX iTOW to UTC time
 
         :param iTOW: int:
         :return datetime.time:
@@ -643,7 +706,8 @@ class UBXMessage:
 
     @staticmethod
     def gpsfix2str(fix: int) -> str:
-        """Converts GPS fix integer to string
+        """
+        Convert GPS fix integer to string
 
         :param fix: int:
         :return str:
@@ -666,7 +730,8 @@ class UBXMessage:
 
     @staticmethod
     def dop2str(dop: float) -> str:
-        """Converts DOP float to descriptive string
+        """
+        Convert Dilution of Precision float to descriptive string
 
         :param dop: float:
         :return str:
@@ -689,7 +754,8 @@ class UBXMessage:
 
     @staticmethod
     def gnss2str(gnssId: int) -> str:
-        """Converts GNSS ID to descriptive string
+        """
+        Convert GNSS ID to descriptive string
 
         :param gnssId: int:
         :return str:
@@ -703,7 +769,8 @@ class UBXMessage:
 
     @staticmethod
     def nmeaver2str(nmeaVersion: int) -> str:
-        """Converts NMEA version integer to readable string
+        """
+        Convert NMEA version integer to readable string
 
         :param nmeaVersion: int:
         :return str:
@@ -715,13 +782,16 @@ class UBXMessage:
 
     @staticmethod
     def mode2str(mode: int) -> str:
-        """Convert mode to string"""
+        """
+        Convert mode to string
+        """
 
         return ["GET", "SET", "POLL"][mode]
 
     @staticmethod
     def key_from_val(dictionary: dict, value) -> str:
-        """Helper method - get dictionary key corresponding to (unique) value.
+        """
+        Helper method - get dictionary key corresponding to (unique) value.
 
         :param dictionary: dict:
         :param value:
@@ -734,3 +804,153 @@ class UBXMessage:
             if val == value:
                 return key
         raise ube.UBXMessageError(f"Undefined message type {value}")
+
+    @staticmethod
+    def cfgname2key(name: str) -> (int, str):
+        """
+        Return hexadecimal key and data type for given
+        configuration database key name.
+
+        :param name: str
+        :return (key: int, type: str)
+        """
+        try:
+            return ubcdb.UBX_CONFIG_DATABASE[name]
+        except KeyError as err:
+            raise ube.UBXMessageError(
+                f"Undefined configuration database key {name}"
+            ) from err
+
+    @staticmethod
+    def cfgkey2name(keyID: int) -> (str, str):
+        """
+        Return key name and data type for given
+        configuration database hexadecimal key.
+
+        :param keyID: int:
+        :return (keyname: str, type: str)
+
+        """
+
+        val = None
+        for key, val in ubcdb.UBX_CONFIG_DATABASE.items():
+            (kid, typ) = val
+            if keyID == kid:
+                return (key, typ)
+        raise ube.UBXMessageError(f"Undefined configuration database key {hex(keyID)}")
+
+    @staticmethod
+    def config_set(layers: int, transaction: int, cfgData: list) -> object:
+        """
+        Construct CFG-VALSET message from an array of
+        configuration database (key, value) tuples. Keys
+        can be in int (keyID) or str (keyname) format.
+
+        :param layers: int: (1=RAM, 2=BBR, 4=Flash)
+        :param transaction: int: (0=no txn, 1=start txn, 2=continue txn, 3=apply txn)
+        :param cfgData: list: (key, value) tuples; max 64 tuples)
+        :return UBXMessage object: CFG-VALSET message
+
+        """
+
+        num = len(cfgData)
+        if num > 64:
+            raise ube.UBXMessageError(
+                f"Number of configuration tuples {num} exceeds maximum of 64"
+            )
+
+        version = 0 if transaction == 0 else 1
+        version = version.to_bytes(1, byteorder="little", signed=False)
+        layers = layers.to_bytes(1, byteorder="little", signed=False)
+        transaction = transaction.to_bytes(1, byteorder="little", signed=False)
+        reserved0 = b"\x00"
+        payload = version + layers + transaction + reserved0
+        lis = b""
+
+        for cfgItem in cfgData:
+            att = ""
+            (key, val) = cfgItem
+            if isinstance(key, str):  # if key is a string (keyname)
+                (key, att) = UBXMessage.cfgname2key(
+                    key
+                )  # lookup keyID & attribute type
+            else:
+                (_, att) = UBXMessage.cfgkey2name(key)  # lookup attribute type
+            keyb = key.to_bytes(4, byteorder="little", signed=False)
+            valb = UBXMessage.val2bytes(val, att)
+            lis = lis + keyb + valb
+
+        return UBXMessage("CFG", "CFG-VALSET", ubt.SET, payload=payload + lis)
+
+    @staticmethod
+    def config_del(layers: int, transaction: int, keys: list) -> object:
+        """
+        Construct CFG-VALDEL message from an array of
+        configuration database keys, which can be in int (keyID)
+        or str (keyname) format.
+
+        :param layers: int: (2=BBR, 4=Flash)
+        :param transaction: int: (0=no txn, 1=start txn, 2=continue txn, 3=apply txn)
+        :param keys: list: keyID as int or keyname as string; max 64 keys
+        :return UBXMessage object: CFG-VALDEL message
+
+        """
+
+        num = len(keys)
+        if num > 64:
+            raise ube.UBXMessageError(
+                f"Number of configuration keys {num} exceeds maximum of 64"
+            )
+
+        version = 0 if transaction == 0 else 1
+        version = version.to_bytes(1, byteorder="little", signed=False)
+        layers = layers.to_bytes(1, byteorder="little", signed=False)
+        transaction = transaction.to_bytes(1, byteorder="little", signed=False)
+        reserved0 = b"\x00"
+        payload = version + layers + transaction + reserved0
+        lis = b""
+
+        for key in keys:
+            if isinstance(key, str):  # if keyname as a string
+                (key, _) = UBXMessage.cfgname2key(key)  # lookup keyID
+            keyb = key.to_bytes(4, byteorder="little", signed=False)
+            lis = lis + keyb
+
+        return UBXMessage("CFG", "CFG-VALDEL", ubt.SET, payload=payload + lis)
+
+    @staticmethod
+    def config_poll(layers: int, position: int, keys: list) -> object:
+        """
+        Construct CFG-VALGET message from an array of
+        configuration database keys, which can be in int (keyID)
+        or str (keyname) format. keyID format allows for wildcard
+        queries.
+
+        :param version: int:
+        :param layers: int: (0=RAM, 1=BBR, 2=Flash, 7 = Default)
+        :param position: int: (number of keys to skip before returning result)
+        :param keys: list: keyID as int or keyname as string; max 64 keys
+        :return UBXMessage object: CFG-VALGET message
+
+        """
+
+        num = len(keys)
+        if num > 64:
+            raise ube.UBXMessageError(
+                f"Number of configuration keys {num} exceeds maximum of 64"
+            )
+
+        version = 0
+        version = version.to_bytes(1, byteorder="little", signed=False)
+        layers = layers.to_bytes(1, byteorder="little", signed=False)
+        position = position.to_bytes(2, byteorder="little", signed=False)
+        payload = version + layers + position
+        lis = b""
+
+        for key in keys:
+            if isinstance(key, str):  # if keyname as a string
+                (key, _) = UBXMessage.cfgname2key(key)  # lookup keyID
+            keyb = key.to_bytes(4, byteorder="little", signed=False)
+            lis = lis + keyb
+
+        return UBXMessage("CFG", "CFG-VALGET", ubt.POLL, payload=payload + lis)
