@@ -2,8 +2,8 @@
 """
 UBX Protocol core globals and constants
 
-Based on u-blox Generation 8 receiver datasheet:
-https://www.u-blox.com/en/docs/UBX-13003221
+Based on u-blox Generation 9 SPG receiver datasheet:
+https://www.u-blox.com/en/docs/UBX-190359401
 
 Created on 27 Sep 2020
 
@@ -30,31 +30,39 @@ GNSSLIST = {
 """
 THESE ARE THE UBX PROTOCOL PAYLOAD ATTRIBUTE TYPES
 """
-U1 = "U01"  # Unsigned Char
-U2 = "U02"  # Unsigned Short
-U3 = "U03"  # Unsigned Long 3
-U4 = "U04"  # Unsigned Long 4
-U5 = "U05"  # Unsigned Long 5
-U6 = "U06"  # Unsigned Long 6
-U8 = "U08"  # Unsigned Long 8
-U12 = "U12"  # Unsigned Long 12
-U40 = "U40"  # Unsigned Long 40
-U64 = "U64"  # Unsigned Long 64
-I1 = "I01"  # Signed Char 1 2's complement
-I2 = "I02"  # Signed Short
-I4 = "I04"  # Signed Long 4 2's complement
-X1 = "X01"  # Bitfield 1
-X2 = "X02"  # Bitfield 2
-X4 = "X04"  # Bitfield 4
-X6 = "X06"  # Bitfield 6
-R4 = "R04"  # Float (IEEE 754) Single Precision 4
-R8 = "R08"  # Float (IEEE 754) Double Precision 8
-C2 = "C02"  # ASCII / ISO 8859.1 Encoding 2
-C6 = "C06"  # ASCII / ISO 8859.1 Encoding 6
-C10 = "C10"  # ASCII / ISO 8859.1 Encoding 10
-C30 = "C30"  # ASCII / ISO 8859.1 Encoding 30
-C32 = "C32"  # ASCII / ISO 8859.1 Encoding 32
+U1 = "U01"  # Unsigned Int 1 byte
+U2 = "U02"  # Unsigned Int 2 bytes
+U3 = "U03"  # Unsigned Int 3 bytes
+U4 = "U04"  # Unsigned Int 4 bytes
+U5 = "U05"  # Unsigned Int 5 bytes
+U6 = "U06"  # Unsigned Int 6 bytes
+U8 = "U08"  # Unsigned Int 8 bytes
+U9 = "U09"  # Unsigned Int 9 bytes
+U12 = "U12"  # Unsigned Int 12 bytes
+U40 = "U40"  # Unsigned Int 40 bytes
+U64 = "U64"  # Unsigned Int 64 bytes
+I1 = "I01"  # Signed Int 2's complement 1 byte
+I2 = "I02"  # Signed Int 2's complement 2 bytes
+I4 = "I04"  # Signed Int 2's complement 4 bytes
+I8 = "I08"  # Signed Int 2's complement 8 bytes
+X1 = "X01"  # Bitfield 1 byte
+X2 = "X02"  # Bitfield 2 bytes
+X4 = "X04"  # Bitfield 4 bytes
+X6 = "X06"  # Bitfield 6 bytes
+X8 = "X08"  # Bitfield 8 bytes
+R4 = "R04"  # Float (IEEE 754) Single Precision 4 bytes
+R8 = "R08"  # Float (IEEE 754) Double Precision 8 bytes
+C2 = "C02"  # ASCII / ISO 8859.1 Encoding 2 bytes
+C6 = "C06"  # ASCII / ISO 8859.1 Encoding 6 bytes
+C10 = "C10"  # ASCII / ISO 8859.1 Encoding 10 bytes
+C30 = "C30"  # ASCII / ISO 8859.1 Encoding 30 bytes
+C32 = "C32"  # ASCII / ISO 8859.1 Encoding 32 bytes
 CH = "CH"  # ASCII / ISO 8859.1 Encoding Variable Length
+E1 = "E01"  # Unsigned Int Enumeration 1 byte
+E2 = "E02"  # Unsigned Int Enumeration 2 bytes
+E4 = "E04"  # Unsigned Int Enumeration 4 bytes
+L = "L01"  # Boolean stored as U01
+
 VALID_TYPES = (
     U1,
     U2,
@@ -63,16 +71,19 @@ VALID_TYPES = (
     U5,
     U6,
     U8,
+    U9,
     U12,
     U40,
     U64,
     I1,
     I2,
     I4,
+    I8,
     X1,
     X2,
     X4,
     X6,
+    X8,
     R4,
     R8,
     C2,
@@ -81,6 +92,10 @@ VALID_TYPES = (
     C30,
     C32,
     CH,
+    E1,
+    E2,
+    E4,
+    L,
 )
 
 """
@@ -100,6 +115,7 @@ UBX_CLASSES = {
     b"\x13": "MGA",  # Multiple GNSS Assistance Messages: Assistance data for various GNSS
     b"\x21": "LOG",  # Logging Messages: Log creation, deletion, info and retrieval
     b"\x27": "SEC",  # Security Feature Messages
+    b"\x28": "HNR",  # High Rate Navigation Messages
     b"\x66": "FOO",  # Dummy message class for testing
 }
 
@@ -110,15 +126,21 @@ Payloads for each of these identities are defined in the ubxtypes_* modules
 UBX_MSGIDS = {
     b"\x05\x01": "ACK-ACK",
     b"\x05\x00": "ACK-NAK",
-    # ***************************************************************
-    # AID messages are deprecated since Gen 8 in favour of MGA
-    # ***************************************************************
+    # *********************************************************************
+    # AssistNow Aiding messages
+    # Since Gen 8, these are deprecated in favour of MGA
+    # *********************************************************************
     b"\x0b\x30": "AID-ALM",
     b"\x0b\x33": "AID-AOP",
     b"\x0b\x31": "AID-EPH",
     b"\x0b\x02": "AID-HUI",
     b"\x0b\x01": "AID-INI",
+    # *********************************************************************
+    # Configuration messages
+    # Since Gen 9, many of these are deprecated in favour of CFG-VALSET/DEL
+    # *********************************************************************
     b"\x06\x13": "CFG-ANT",
+    b"\x06\x93": "CFG-BATCH",
     b"\x06\x09": "CFG-CFG",
     b"\x06\x06": "CFG-DAT",
     b"\x06\x61": "CFG-DOSC",
@@ -149,24 +171,49 @@ UBX_MSGIDS = {
     b"\x06\x31": "CFG-TP5",
     b"\x06\x53": "CFG-TXSLOT",
     b"\x06\x1b": "CFG-USB",
+    b"\x06\x8c": "CFG-VALDEL",
+    b"\x06\x8b": "CFG-VALGET",
+    b"\x06\x8a": "CFG-VALSET",
+    # ***************************************************************
+    # External Sensor Fusion messages
+    # ***************************************************************
+    b"\x10\x14": "ESF-ALG",
+    b"\x10\x15": "ESF-INS",
+    b"\x10\x02": "ESF-MEAS",
+    b"\x10\x03": "ESF-RAW",
     b"\x10\x10": "ESF-STATUS",
+    # ***************************************************************
+    # High Rate Navigation messages
+    # ***************************************************************
+    b"\x28\x01": "HNR-ATT",
+    b"\x28\x02": "HNR-INS",
+    b"\x28\x00": "HNR-PVT",
+    # ***************************************************************
+    # Information messages
+    # ***************************************************************
     b"\x04\x04": "INF-DEBUG",
     b"\x04\x00": "INF-ERROR",
     b"\x04\x02": "INF-NOTICE",
     b"\x04\x03": "INF-TEST",
     b"\x04\x01": "INF-WARNING",
+    # ***************************************************************
+    # Logging messages
+    # ***************************************************************
+    b"\x21\x11": "LOG-BATCH",
     b"\x21\x07": "LOG-CREATE",
     b"\x21\x03": "LOG-ERASE",
     b"\x21\x0e": "LOG-FINDTIME",
     b"\x21\x08": "LOG-INFO",
-    b"\x21\x0f": "LOG-RETRIEVEPOSEXTRA",
-    b"\x21\x0b": "LOG-RETRIEVEPOS",
-    b"\x21\x0d": "LOG-RETRIEVESTRING",
     b"\x21\x09": "LOG-RETRIEVE",
+    b"\x21\x10": "LOG-RETRIEVEBATCH",
+    b"\x21\x0b": "LOG-RETRIEVEPOS",
+    b"\x21\x0f": "LOG-RETRIEVEPOSEXTRA",
+    b"\x21\x0d": "LOG-RETRIEVESTRING",
     b"\x21\x04": "LOG-STRING",
     # ***************************************************************
-    # MGA messages need special handling as MSGIDs are not unique
-    # Message identity is determined by 'type' attribute in payload
+    # Multiple GNSS Assistance messages
+    # These need special handling as MSGIDs alone are not unique;
+    # message identity is determined by 'type' attribute in payload
     # ***************************************************************
     b"\x13\x60\x01": "MGA-ACK-DATA0",
     b"\x13\x60\x00": "MGA-NAK-DATA0",
@@ -202,6 +249,10 @@ UBX_MSGIDS = {
     b"\x13\x05\x01": "MGA-QZSS-EPH",
     b"\x13\x05\x02": "MGA-QZSS-ALM",
     b"\x13\x05\x04": "MGA-QZSS-HEALTH",
+    # ***************************************************************
+    # Hardware Monitoring messages
+    # ***************************************************************
+    b"\x0a\x36": "MON-COMMS",
     b"\x0a\x28": "MON-GNSS",
     b"\x0a\x0b": "MON-HW2",
     b"\x0a\x09": "MON-HW",
@@ -211,42 +262,66 @@ UBX_MSGIDS = {
     b"\x0a\x07": "MON-RXBUF",
     b"\x0a\x21": "MON-RXR",
     b"\x0a\x2e": "MON-SMGR",
+    b"\x0a\x31": "MON-SPAN",
     b"\x0a\x08": "MON-TXBUF",
     b"\x0a\x04": "MON-VER",
+    # ***************************************************************
+    # Navigation messages
+    # ***************************************************************
     b"\x01\x60": "NAV-AOPSTATUS",
     b"\x01\x22": "NAV-CLOCK",
+    b"\x01\x36": "NAV-COV",
     b"\x01\x31": "NAV-DGPS",
     b"\x01\x04": "NAV-DOP",
+    b"\x01\x3d": "NAV-EELL",
     b"\x01\x61": "NAV-EOE",
     b"\x01\x39": "NAV-GEOFENCE",
+    b"\x01\x13": "NAV-HPPOSECEF",
+    b"\x01\x14": "NAV-HPPOSLLH",
+    b"\x01\x28": "NAV-NMI",
     b"\x01\x09": "NAV-ODO",
     b"\x01\x34": "NAV-ORB",
     b"\x01\x01": "NAV-POSECEF",
     b"\x01\x02": "NAV-POSLLH",
     b"\x01\x07": "NAV-PVT",
+    b"\x01\x3c": "NAV-RELPOSNED",
     b"\x01\x10": "NAV-RESETODO",
     b"\x01\x35": "NAV-SAT",
     b"\x01\x32": "NAV-SBAS",
+    b"\x01\x43": "NAV-SIG",
+    b"\x01\x42": "NAV-SLAS",
     b"\x01\x06": "NAV-SOL",
     b"\x01\x03": "NAV-STATUS",
     b"\x01\x30": "NAV-SVINFO",
+    b"\x01\x3b": "NAV-SVIN",
     b"\x01\x24": "NAV-TIMEBDS",
     b"\x01\x25": "NAV-TIMEGAL",
     b"\x01\x23": "NAV-TIMEGLO",
     b"\x01\x20": "NAV-TIMEGPS",
     b"\x01\x26": "NAV-TIMELS",
+    b"\x01\x27": "NAV-TIMEQZSS",
     b"\x01\x21": "NAV-TIMEUTC",
     b"\x01\x11": "NAV-VELECEF",
     b"\x01\x12": "NAV-VELNED",
+    # ***************************************************************
+    # Receiver Management messages
+    # ***************************************************************
     b"\x02\x61": "RXM-IMES",
     b"\x02\x14": "RXM-MEASX",
     b"\x02\x41": "RXM-PMREQ",
     b"\x02\x15": "RXM-RAWX",
     b"\x02\x59": "RXM-RLM",
+    b"\x02\x32": "RXM-RTCM",
     b"\x02\x13": "RXM-SFRBX",
     b"\x02\x20": "RXM-SVSI",
+    # ***************************************************************
+    # Security messages
+    # ***************************************************************
     b"\x27\x01": "SEC-SIGN",
     b"\x27\x03": "SEC-UNIQID",
+    # ***************************************************************
+    # Timing messages
+    # ***************************************************************
     b"\x0d\x11": "TIM-DOSC",
     b"\x0d\x16": "TIM-FCHG",
     b"\x0d\x17": "TIM-HOC",
@@ -257,6 +332,9 @@ UBX_MSGIDS = {
     b"\x0d\x01": "TIM-TP",
     b"\x0d\x15": "TIM-VCOCAL",
     b"\x0d\x06": "TIM-VRFY",
+    # ***************************************************************
+    # Firmware update messages
+    # ***************************************************************
     b"\x09\x14": "UPD-SOS",
     b"\x66\x66": "FOO-BAR",  # Dummy message for testing
 }
@@ -290,8 +368,10 @@ UBX_CONFIG_MESSAGES = {
     b"\x01\x60": "NAV-AOPSTATUS",
     b"\x01\x05": "NAV-ATT",
     b"\x01\x22": "NAV-CLOCK",
+    b"\x01\x36": "NAV-COV",
     b"\x01\x31": "NAV-DGPS",
     b"\x01\x04": "NAV-DOP",
+    b"\x01\x3d": "NAV-EELL",
     b"\x01\x40": "NAV-EKFSTATUS",
     b"\x01\x61": "NAV-EOE",
     b"\x01\x39": "NAV-GEOFENCE",
@@ -326,8 +406,10 @@ UBX_CONFIG_MESSAGES = {
     b"\x02\x41": "RXM-PMREQ",
     b"\x02\x15": "RXM-RAWX",
     b"\x02\x59": "RXM-RLM",
+    b"\x02\x32": "RXM-RTCM",
     b"\x02\x13": "RXM-SFRBX",
     b"\x02\x20": "RXM-SVSI",
+    b"\x0a\x36": "MON-COMMS",
     b"\x0a\x28": "MON-GNSS",
     b"\x0a\x09": "MON-HW",
     b"\x0a\x0b": "MON-HW2",
@@ -339,9 +421,10 @@ UBX_CONFIG_MESSAGES = {
     b"\x0a\x07": "MON-RXBUF",
     b"\x0a\x21": "MON-RXR",
     b"\x0a\x2e": "MON-SMGR",
-    # b'\x0a\x31': 'MON-SPAN',
+    b"\x0a\x31": "MON-SPAN",
     b"\x0a\x08": "MON-TXBUF",
     b"\x0a\x04": "MON-VER",
+    b"\x21\x11": "LOG-BATCH",
     b"\x21\x07": "LOG-CREATE",
     b"\x21\x03": "LOG-ERASE",
     b"\x21\x0e": "LOG-FINDTIME",
@@ -350,7 +433,14 @@ UBX_CONFIG_MESSAGES = {
     b"\x21\x0b": "LOG-RETRIEVEPOS",
     b"\x21\x0d": "LOG-RETRIEVESTRING",
     b"\x21\x09": "LOG-RETRIEVE",
+    b"\x21\x10": "LOG-RETRIEVEBATCH",
     b"\x21\x04": "LOG-STRING",
+    b"\x28\x01": "HNR-ATT",
+    b"\x28\x02": "HNR-INS",
+    b"\x28\x00": "HNR-PVT",
+    b"\x10\x14": "ESF-ALG",
+    b"\x10\x15": "ESF-INS",
+    b"\x10\x10": "ESF-STATUS",
     b"\xf0\x0a": "DTM",  # Datum Reference
     b"\xf0\x44": "GBQ",  # Poll Standard Message - Talker ID GB (BeiDou)
     b"\xf0\x09": "GBS",  # GNSS Satellite Fault Detection
