@@ -21,22 +21,23 @@ Created on 27 Sep 2020
 
 from pyubx2.ubxtypes_core import (
     U1,
-    I1,
-    X1,
     U2,
-    I2,
-    X2,
     U3,
     U4,
-    I4,
     U5,
-    X4,
-    R4,
     U6,
-    R8,
     U8,
+    U9,
     U12,
     U64,
+    I1,
+    I2,
+    I4,
+    X1,
+    X2,
+    X4,
+    R4,
+    R8,
     C2,
     C6,
     C10,
@@ -52,7 +53,6 @@ UBX_PAYLOADS_GET = {
     # AssistNow Aiding Messages: i.e. Ephemeris, Almanac, other A-GPS data input.
     # Messages in the AID class are used to send GPS aiding data to the receiver
     # AID messages are deprecated in favour of MGA messages in >=Gen8
-    # AID messages are deprecated in favour of MGA messages in >=Gen8
     "AID-ALM": {"svid": U4, "week": U4, "optBlock": ("None", {"dwrd": U4})},
     "AID-AOP": {"gnssId": U1, "svId": U1, "reserved1": U2, "data": U64},
     "AID-EPH": {
@@ -130,6 +130,14 @@ UBX_PAYLOADS_GET = {
     # messages in the CFG class sent to the receiver are either acknowledged (with message UBX-ACK-ACK) if
     # processed successfully or rejected (with message UBX-ACK-NAK) if processing unsuccessfully.
     "CFG-ANT": {"flags": X2, "pins": X2},
+    "CFG-BATCH": {
+        "version": U1,
+        "flags": X1,
+        "bufSize": U2,
+        "notifThrs": U2,
+        "pioId": U1,
+        "reserved0": U1,
+    },
     "CFG-CFG": {"clearMask": X4, "saveMask": X4, "loadMask": X4, "deviceMask": X1},
     "CFG-DAT": {
         "datumNum": U2,
@@ -446,9 +454,69 @@ UBX_PAYLOADS_GET = {
         "productString": C32,
         "serialNumber": C32,
     },
+    "CFG-VALGET": {
+        "version": U1,
+        "layer": U1,
+        "position": U2,
+        "group": ("None", {"cfgData": U1}),  # repeating group
+    },
     # ********************************************************************
     # External Sensor Fusion Messages: i.e. External Sensor Measurements and Status Information.
     # Messages in the ESF class are used to output external sensor fusion information from the receiver.
+    "ESF-ALG": {
+        "iTOW": U4,
+        "version": U1,
+        "flags": U1,
+        "error": U1,
+        "reserved1": U1,
+        "yaw": U4,
+        "pitch": I2,
+        "roll": I2,
+    },
+    "ESF-INS": {
+        "bitfield0": U4,
+        "reserved1": U4,
+        "iTOW": U4,
+        "xAngRate": I4,
+        "yAngRate": I4,
+        "zAngRate": I4,
+        "xAccel": I4,
+        "yAccel": I4,
+        "zAccel": I4,
+    },
+    "ESF-MEAS": {  # this version used when bit 3 of flags = 0
+        "timeTag": U4,
+        "flags": X2,
+        "id": U2,
+        "group": (
+            "None",
+            {  # repeating group * numMeas, which is bits 11..15 in flags
+                "data": X4,
+            },
+        ),
+    },
+    "ESF-MEAS-CT": {  # this version used when bit 3 of flags = 1
+        "timeTag": U4,
+        "flags": X2,
+        "id": U2,
+        "group": (
+            "ESF-MEAS-CT",
+            {  # repeating group * numMeas, which is bits 11..15 of flags
+                "data": X4,
+            },
+        ),
+        "calibTtag": U4,
+    },
+    "ESF-RAW": {
+        "reserved1": U4,
+        "group": (
+            "None",
+            {  # repeating group
+                "data": X4,
+                "sTag": X4,
+            },
+        ),
+    },
     "ESF-STATUS": {
         "iTOW": U4,
         "version": U1,
@@ -470,6 +538,58 @@ UBX_PAYLOADS_GET = {
         ),
     },
     # ********************************************************************
+    # High Rate Navigation Results Messages: i.e. High rate time, position, speed, heading.
+    # Messages in the HNR class are used to output high rate navigation data for position, altitude,
+    # velocity and their accuracies.
+    "HNR-ATT": {
+        "iTOW": U4,
+        "version": U1,
+        "reserved1": U3,
+        "roll": I4,
+        "pitch": I4,
+        "heading": I4,
+        "accRoll": U4,
+        "accPitch": U4,
+        "accHeading": U4,
+    },
+    "HNR-INS": {
+        "bitfield0": X4,
+        "reserved1": U4,
+        "iTOW": U4,
+        "xAngRate": I4,
+        "yAngRate": I4,
+        "zAngRate": I4,
+        "xAccel": I4,
+        "yAccel": I4,
+        "zAccel": I4,
+    },
+    "HNR-PVT": {
+        "iTOW": U4,
+        "year": U2,
+        "month": U1,
+        "day": U1,
+        "hour": U1,
+        "min": U1,
+        "second": U1,
+        "valid": X1,
+        "nano": I4,
+        "flags": X1,
+        "reserved1": U2,
+        "lon": I4,
+        "lat": I4,
+        "height": I4,
+        "hMSL": I4,
+        "gSpeed": I4,
+        "speed": I4,
+        "headMot": I4,
+        "headVeh": I4,
+        "hAcc": U4,
+        "vAcc": U4,
+        "sAcc": U4,
+        "headAcc": U4,
+        "reserved2": U4,
+    },
+    # ********************************************************************
     # Information Messages: i.e. Printf-Style Messages, with IDs such as Error, Warning, Notice.
     # Messages in the INF class are used to output strings in a printf style from the firmware or application code. All
     # INF messages have an associated type to indicate the kind of message.
@@ -519,13 +639,6 @@ UBX_PAYLOADS_GET = {
         "distanceStd": U4,
         "reserved2": U4,
     },
-    "LOG-CREATE": {
-        "version": U1,
-        "logCfg": X1,
-        "reserved1": U1,
-        "logSize": U1,
-        "userDefinedSize": U4,
-    },
     "LOG-FINDTIME": {"version": U1, "type": U1, "reserved1": U2, "entryNumber": U4},
     "LOG-INFO": {
         "version": U1,
@@ -552,20 +665,6 @@ UBX_PAYLOADS_GET = {
         "status": X1,
         "reserved5": U3,
     },
-    "LOG-RETRIEVEPOSEXTRA": {
-        "entryIndex": U4,
-        "version": U1,
-        "reserved1": U1,
-        "year": U2,
-        "month": U1,
-        "day": U1,
-        "hour": U1,
-        "minute": U1,
-        "second": U1,
-        "reserved2": U3,
-        "distance": U4,
-        "reserved3": U12,
-    },
     "LOG-RETRIEVEPOS": {
         "entryIndex": U4,
         "lon": I4,
@@ -586,6 +685,20 @@ UBX_PAYLOADS_GET = {
         "numSV": U1,
         "reserved2": U1,
     },
+    "LOG-RETRIEVEPOSEXTRA": {
+        "entryIndex": U4,
+        "version": U1,
+        "reserved1": U1,
+        "year": U2,
+        "month": U1,
+        "day": U1,
+        "hour": U1,
+        "minute": U1,
+        "second": U1,
+        "reserved2": U3,
+        "distance": U4,
+        "reserved3": U12,
+    },
     "LOG-RETRIEVESTRING": {
         "entryIndex": U4,
         "version": U1,
@@ -629,6 +742,37 @@ UBX_PAYLOADS_GET = {
     # Monitoring Messages: i.e. Communication Status, CPU Load, Stack Usage, Task Status.
     # Messages in the MON class are used to report the receiver status, such as CPU load, stack usage, I/O subsystem
     # statistics etc.
+    "MON-COMMS": {
+        "version": U1,
+        "nPorts": U1,
+        "txErrors": X1,
+        "reserved0": U1,
+        "protIds1": U1,
+        "protIds2": U1,
+        "protIds3": U1,
+        "protIds4": U1,
+        "group": (
+            "nPorts",
+            {  # repeating group * nPorts
+                "portId": U2,
+                "txPending": U2,
+                "txBytes": U4,
+                "txUsage": U1,
+                "txPeakUsage": U1,
+                "rxPending": U2,
+                "rxBytes": U4,
+                "rxUsage": U1,
+                "rxPeakUsage": U1,
+                "overrunErrs": U2,
+                "msgs1": U2,
+                "msgs2": U2,
+                "msgs3": U2,
+                "msgs4": U2,
+                "reserved1": U8,
+                "skipped": U4,
+            },
+        ),
+    },
     "MON-GNSS": {
         "version": U1,
         "supported": X1,
@@ -805,6 +949,25 @@ UBX_PAYLOADS_GET = {
         "extInt0": X1,
         "extInt1": X1,
     },
+    "MON-SPAN": {
+        "version": U1,
+        "numRfBlocks": U1,
+        "reserved0": U2,
+        "group": (
+            "numRfBlocks",
+            {  # repeating group * numRfBlocks
+                "spectrum001_064": U64,
+                "spectrum065_128": U64,
+                "spectrum129_192": U64,
+                "spectrum193_256": U64,
+                "span": U4,
+                "res": U4,
+                "center": U4,
+                "pga": U1,
+                "reserved1": U3,
+            },
+        ),
+    },
     "MON-TXBUF": {
         "pending0": U2,
         "pending1": U2,
@@ -850,6 +1013,25 @@ UBX_PAYLOADS_GET = {
         "reserved3": U4,
     },
     "NAV-CLOCK": {"iTOW": U4, "clkB": I4, "clkD": I4, "tAcc": U4, "fAcc": U4},
+    "NAV-COV": {
+        "iTOW": U4,
+        "version": U1,
+        "posCovValid": U1,
+        "velCovValid": U1,
+        "reserved0": U9,
+        "posCovNN": R4,
+        "posCovNE": R4,
+        "posCovND": R4,
+        "posCovEE": R4,
+        "posCovED": R4,
+        "posCovDD": R4,
+        "velCovNN": R4,
+        "velCovNE": R4,
+        "velCovND": R4,
+        "velCovEE": R4,
+        "velCovED": R4,
+        "velCovDD": R4,
+    },
     "NAV-DGPS": {
         "iTOW": U4,
         "age": I4,
@@ -879,6 +1061,14 @@ UBX_PAYLOADS_GET = {
         "nDOP": U2,
         "eDOP": U2,
     },
+    "NAV-EELL": {
+        "iTOW": U4,
+        "version": U1,
+        "reserved1": U1,
+        "errEllipseOrient": U2,
+        "errEllipseMajor": U4,
+        "errEllipseMinor": U4,
+    },
     "NAV-EKFSTATUS": {
         "pulses": I4,
         "period": I4,
@@ -898,9 +1088,50 @@ UBX_PAYLOADS_GET = {
             {"state": U1, "reserved1": U1},  # repeating group * numFences
         ),
     },
+    "NAV-HPPOSECEF": {
+        "version": U1,
+        "reserved1": U3,
+        "iTOW": U4,
+        "ecefX": I4,
+        "ecefY": I4,
+        "ecefZ": I4,
+        "ecefXHp": I1,
+        "ecefYHp": I1,
+        "ecefZHp": I1,
+        "reserved2": U1,
+        "pAcc": U4,
+    },
+    "NAV-HPPOSLLH": {
+        "version": U1,
+        "reserved": U2,
+        "flags": X1,
+        "iTOW": U4,
+        "lon": I4,
+        "lat": I4,
+        "height": I4,
+        "hMSL": I4,
+        "lonHp": I1,
+        "latHp": I1,
+        "heightHp": I1,
+        "hMSLHp": I1,
+        "hAcc": U4,
+        "vAcc": U4,
+    },
+    "NAV-NMI": {
+        "iTOW": U4,
+        "version": U1,
+        "reserved1": U4,
+        "gpsNmiFlags": X1,
+        "gpsLsFlags": X1,
+        "galNmiFlags": X1,
+        "galLsFlags": X1,
+        "bdsNmiFlags": X1,
+        "bdsLsFlags": X1,
+        "gloNmiFlags": X1,
+    },
     "NAV-ODO": {
         "version": U1,
-        "reserved13": U1,
+        "reserved0": U3,
         "iTOW": U4,
         "distance": U4,
         "totalDistance": U4,
@@ -909,11 +1140,11 @@ UBX_PAYLOADS_GET = {
     "NAV-ORB": {
         "iTOW": U4,
         "version": U1,
-        "numCh": U1,
+        "numSv": U1,
         "reserved1": U2,
         "group": (
-            "numCh",
-            {  # repeating group * numCh
+            "numSv",
+            {  # repeating group * numSv
                 "gnssId": U1,
                 "svId": U1,
                 "svFlag": X1,
@@ -967,6 +1198,23 @@ UBX_PAYLOADS_GET = {
         "magDec": I2,
         "magAcc": U2,
     },
+    "NAV-RELPOSNED": {
+        "version": U1,
+        "reserved1": U1,
+        "refStationID": U2,
+        "iTOW": U4,
+        "relPosN": I4,
+        "relPosE": I4,
+        "relPosD": I4,
+        "relPosHPN": I1,
+        "relPosHPE": I1,
+        "relPosHPD": I1,
+        "reserved2": U1,
+        "accN": U4,
+        "accE": U4,
+        "accD": U4,
+        "flags": X4,
+    },
     "NAV-SAT": {
         "iTOW": U4,
         "version": U1,
@@ -1009,6 +1257,49 @@ UBX_PAYLOADS_GET = {
             },
         ),
     },
+    "NAV-SIG": {
+        "iTOW": U4,
+        "version": U1,
+        "numSigs": U1,
+        "reserved0": U2,
+        "group": (
+            "numSigs",
+            {  # repeating group * numSigs
+                "gnssId": U1,
+                "svId": U1,
+                "sigId": U1,
+                "freqId": U1,
+                "prRes": I2,
+                "cno": U1,
+                "qualityInd": U1,
+                "corrSource": U1,
+                "ionoModel": U1,
+                "sigFlags": X2,
+                "reserved1": U4,
+            },
+        ),
+    },
+    "NAV-SLAS": {
+        "iTOW": U4,
+        "version": U1,
+        "reserved1": U3,
+        "gmsLon": I4,
+        "gmsLat": I4,
+        "gmsCode": U1,
+        "qzssSvId": U1,
+        "serviceFlags": X1,
+        "cnt": U1,
+        "group": (
+            "cnt",
+            {  # repeating group * cnt
+                "gnssID": U1,
+                "svId": U1,
+                "reserved2": U1,
+                "reserved3": U3,
+                "prc": I2,
+            },
+        ),
+    },
     "NAV-SOL": {
         "iTOW": U4,
         "fTOW": I4,
@@ -1056,6 +1347,24 @@ UBX_PAYLOADS_GET = {
             },
         ),
     },
+    "NAV-SVIN": {
+        "version": U1,
+        "reserved1": U3,
+        "iTOW": U4,
+        "dur": U4,
+        "meanX": I4,
+        "meanY": I4,
+        "meanZ": I4,
+        "meanXHP": I1,
+        "meanYHP": I1,
+        "meanZHP": I1,
+        "reserved2": U1,
+        "meanAcc": U4,
+        "obs": U4,
+        "valid": U1,
+        "active": U1,
+        "reserved3": U2,
+    },
     "NAV-TIMEBDS": {
         "iTOW": U4,
         "SOW": U4,
@@ -1105,6 +1414,15 @@ UBX_PAYLOADS_GET = {
         "reserved2": U3,
         "valid": X1,
     },
+    "NAV-TIMEQZSS": {
+        "iTOW": U4,
+        "qzssTow": U4,
+        "fQzssTow": I4,
+        "qzssWno": I2,
+        "leapS": I1,
+        "valid": X1,
+        "tAcc": U4,
+    },
     "NAV-TIMEUTC": {
         "iTOW": U4,
         "tAcc": U1,
