@@ -121,6 +121,14 @@ UBX_PAYLOADS_SET = {
     # messages in the CFG class sent to the receiver are either acknowledged (with message UBX-ACK-ACK) if
     # processed successfully or rejected (with message UBX-ACK-NAK) if processing unsuccessfully.
     "CFG-ANT": {"flags": X2, "pins": X2},
+    "CFG-BATCH": {
+        "version": U1,
+        "flags": X1,
+        "bufSize": U2,
+        "notifThrs": U2,
+        "pioId": U1,
+        "reserved0": U1,
+    },
     "CFG-CFG": {"clearMask": X4, "saveMask": X4, "loadMask": X4, "deviceMask": X1},
     "CFG-DAT": {
         "datumNum": U2,
@@ -437,16 +445,80 @@ UBX_PAYLOADS_SET = {
         "productString": C32,
         "serialNumber": C32,
     },
+    "CFG-VALDEL": {
+        "version": U1,  # = 0 no transaction, 1 with transaction
+        "layers": X1,
+        "transaction": X1,  # if version = 1, else reserved
+        "reserved0": U1,
+        "group": ("None", {"keys": U4}),  # repeating group
+    },
+    "CFG-VALSET": {
+        "version": U1,  # = 0 no transaction, 1 with transaction
+        "layers": X1,
+        "transaction": U1,  # if version = 1, else reserved
+        "reserved0": U1,
+        "group": ("None", {"cfgData": U1}),  # repeating group
+    },
+    # ********************************************************************
+    # External Sensor Fusion Messages: i.e. External Sensor Measurements and Status Information.
+    # Messages in the ESF class are used to output external sensor fusion information from the receiver.
+    "ESF-MEAS": {  # this version used when bit 3 of flags = 0
+        "timeTag": U4,
+        "flags": X2,
+        "id": U2,
+        "group": (
+            "None",
+            {  # repeating group * numMeas, which is bits 11..15 in flags
+                "data": X4,
+            },
+        ),
+    },
+    "ESF-MEAS-CT": {  # this version used when bit 3 of flags = 1
+        "timeTag": U4,
+        "flags": X2,
+        "id": U2,
+        "group": (
+            "ESF-MEAS-CT",
+            {  # repeating group * numMeas, which is bits 11..15 of flags
+                "data": X4,
+            },
+        ),
+        "calibTtag": U4,
+    },
     # ********************************************************************
     # Logging Messages: i.e. Log creation, deletion, info and retrieval.
     # Messages in the LOG class are used to configure and report status information of the logging feature.
+    "LOG-CREATE": {
+        "version": U1,
+        "logCfg": X1,
+        "reserved1": U1,
+        "logSize": U1,
+        "userDefinedSize": U4,
+    },
     "LOG-ERASE": {},
+    "LOG-FINDTIME": {
+        "version": U1,
+        "type": U1,
+        "reserved0": U2,
+        "year": U2,
+        "month": U1,
+        "day": U1,
+        "hour": U1,
+        "minute": U1,
+        "second": U1,
+        "reserved1": U1,
+    },
     "LOG-RETRIEVE": {
         "startNumber": U4,
         "entryCount": U4,
         "version": U1,
         "reserved": U3,
     },
+    "LOG-RETRIEVEBATCH": {
+        "version": U1,
+        "flags": X1,
+        "reserved0": U2,
+    },
     "LOG-STRING": {"group": ("None", {"bytes": U1})},  # repeating group
     # ********************************************************************
     # Multiple GNSS Assistance Messages: i.e. Assistance data for various GNSS.
